==> taboola_sdk-0.2.1.tar.gz: lib/classic/webview_manager.dart <==
import 'dart:async';
import 'dart:convert';
import 'package:flutter/material.dart';
import 'package:taboola_sdk/descriptors/application_descriptor.dart';
import 'package:taboola_sdk/descriptors/device_descriptor.dart';
import 'package:taboola_sdk/descriptors/sdk_descriptor.dart';
import 'package:taboola_sdk/classic/taboola_bridge.dart';
import 'package:taboola_sdk/utils/rects_utils.dart';
import 'package:taboola_sdk/utils/type_converter_utils.dart';
import 'package:webview_flutter/webview_flutter.dart';
import 'package:taboola_sdk/utils/logger.dart';
import 'package:taboola_sdk/native_plugin/taboola_sdk_plugin.dart';
import '../extra_properties_manager.dart';

const String onRenderSuccess = "onRenderSuccess";
const String onRenderFailure = "onRenderFailure";
const String onClick = "onClick";
const String onAttributionClick = "handleAttributionClick";
const String getDeviceData = "getDeviceData";
const String getLogLevel = "getLogLevel";
const String getExternalRects = "getExternalRects";
const String isAlive = "isAlive";
const String sendNotifyExternalRects = "notifyExternalRects";
const String sendFireClickEvent = "fireClickEvent";
const String webViewDidScrollToTop = "webViewDidScrollToTop";

class WebViewManager {
  Rect webViewRect;
  Rect nativeWindowRect;
  TaboolaBridge taboolaBridge;
  ExtraPropertiesManager _extraPropertiesManager;
  bool _didShow = false;

  // Taboola Classic Callbacks
  void Function(String) didShow;
  void Function(String, double) didResize;
  void Function(String, String) didFailToLoad;
  bool Function(String, String, String, bool) didClickOnItem;
  void Function(String) openItemBrowser;
  void Function() taboolaWebViewDidScrollToTop;

  WebViewManager(this._extraPropertiesManager);

  set controller(WebViewController controller) {
    if (taboolaBridge == null) {
      taboolaBridge = TaboolaBridge(controller);
    }
  }

  JavascriptChannel javascriptChannel() {
    return JavascriptChannel(
        name: "flutterBridge",
        onMessageReceived: (JavascriptMessage result) {
          Map<String, dynamic> messageMap = jsonDecode(result.message);
          String action = messageMap["action"];
          String promiseId = messageMap["promiseId"];
          List parameters = messageMap["parameters"];
//          Logger.log("message ${result.message}");
          _callNativeFunctionFromJS(action, promiseId, parameters);
        });
  }

  Future<void> _callNativeFunctionFromJS(
      String action, String promiseId, List parameters) async {
    Function func;
    switch (action) {
      case onRenderSuccess:
        func = _onRenderSuccess;
        break;
      case onRenderFailure:
        func = _onRenderFailure;
        break;
      case onClick:
        func = _onClick;
        break;
      case onAttributionClick:
        func = _onAttributionClick;
        break;
      case getDeviceData:
        func = _getDeviceData;
        break;
      case getLogLevel:
        func = _getLogLevel;
        break;
      case getExternalRects:
        func = _getExternalRects;
        break;
      case isAlive:
        func = _isAlive;
        break;
      case webViewDidScrollToTop:
        func = _webViewDidScrollToTop;
        break;
      default:
        return;
    }
    try {
      var returnValue = await Function.apply(func, parameters);
      taboolaBridge?.resolvePromise(promiseId, returnValue);
    } catch (e) {
      Logger.logException("$func,$e");
    }
  }

  // JS calling Native
  void _onRenderSuccess(String placement, dynamic height) {
    didResize(placement, TypeConverterUtils.toDouble(height));
    if (!_didShow) {
      _didShow = true;
      didShow(placement);
    }
  }

  void _onRenderFailure(String placement, String error) {
    didFailToLoad(placement, error);
  }

  void _onClick(String title, String pageUrl, String loggerUrl, bool organic,
      String itemId, String placement) {
    bool taboolaHandleClick = true;
    bool publisherInputForHandeClick =
        didClickOnItem(placement, itemId, pageUrl, organic);

    if (organic) {
      taboolaHandleClick = publisherInputForHandeClick;
    }

    //if taboola needs to handle the click, open a browser with the link
    //if not, only fire click event
    if (taboolaHandleClick) {
      openItemBrowser(loggerUrl);
    } else {
      Logger.log(
          '_onClick: Publisher requested SDK not to handle click by returning false in taboolaDidClickOnItem event.');
      _fireClickEvent(loggerUrl);
    }
  }

  void _onAttributionClick(String url) {
    var encoded = Uri.encodeFull(url);
    openItemBrowser(encoded);
  }

  Future<String> _getDeviceData() async {
    List screenSize = DeviceDescriptor.getScreenSize();

    var deviceData = {
      "verifier_enabled": false,
      "additional_data": {
        "sdkd": {
          "sdkt": SdkDescriptor.getSdkName(),
          "sdkv": SdkDescriptor.getSdkVersion(),
          "app": await ApplicationDescriptor.getAppName(),
          "model": await DeviceDescriptor.getDeviceModel(),
          "appv": await ApplicationDescriptor.getAppVersion(),
          "appid": await ApplicationDescriptor.getAppPackageName(),
          "lng": await DeviceDescriptor.getLanguageJson(),
          "os": await DeviceDescriptor.getDeviceOS(),
          "osv": await DeviceDescriptor.getDeviceSystemVersion(),
//          "kotlin": true, //FIXME: Exists in native SDK, implement post MVP
//          "androidx": true, //FIXME: Exists in native SDK, implement post MVP
//          "net": await DeviceDescriptor.getNetworkConnSpeed(), //FIXME: Re-enable once code is ripped from Connectivity plugin
          "mem": await DeviceDescriptor.getDeviceFreeMemory(),
          "scr": {"wd": screenSize[0], "ht": screenSize[1]}
        }
      },
      "device": await DeviceDescriptor.getAdvertisingId(),
//      "allowNonOrganicClickOverride": false, //FIXME: Exists in native SDK, implement post MVP
//      "isUsedInTaboolaWidget": true, //FIXME: Exists in native SDK, implement post MVP
//      "taboolaConfig": "{}" //FIXME: Exists in native SDK, implement post MVP
    };
    await selfPopulateExtraProperties();

    Map<String, dynamic> dataToMobileLoader = new Map<String, dynamic>();
    dataToMobileLoader.addAll(_extraPropertiesManager.getExtraProperties());
    dataToMobileLoader.addAll(deviceData);

    String encoded = json.encode(dataToMobileLoader);
    Logger.log('deviceData: $encoded');

    return encoded;
  }

  Future<void> selfPopulateExtraProperties() async {
    //Consulting product manager, the user opt out will only be calculated by what the user selected in their operating system definitions. Not by publisher.
    _extraPropertiesManager.put(ExtraPropertiesManager.USER_OPT_OUT_KEY,
        await TaboolaSdkPlugin.isUserOptOut());
  }

//  Future<Map<String,dynamic >> _getSdkd() async {
//
//    List screenSize = DeviceDescriptor.getScreenSize();
//
//    Map <String,dynamic> scr = new Map <String,dynamic>();
//    scr["wd"] = screenSize[0];
//    scr["ht"] = screenSize[1];
//
//    Map <String,dynamic> sdkd = new Map <String,dynamic>();
//    sdkd["sdkt"] = SdkDescriptor.getSdkName();
//    sdkd["sdkv"] = SdkDescriptor.getSdkVersion();
//    sdkd["app"] = ApplicationDescriptor.getAppName();
//    sdkd["model"] = DeviceDescriptor.getDeviceModel();
//    sdkd["appv"] = ApplicationDescriptor.getAppVersion();
//    sdkd["appid"] = ApplicationDescriptor.getAppPackageName();
//    sdkd["lng"] = DeviceDescriptor.getLanguageJson() ;
//    sdkd["os"] = DeviceDescriptor.getDeviceOS();
//    sdkd["osv"] = DeviceDescriptor.getDeviceSystemVersion();
//    sdkd["net"] = DeviceDescriptor.getNetworkConnSpeed();
//    sdkd["scr"] = scr;
//
//    Map <String,dynamic> newSdkd = new Map <String,dynamic>();
//
//    sdkd.forEach((k,v) => v.then(newSdkd[k]=v));
//
//    return await newSdkd;
//  }

  String _getLogLevel() {
    if (Logger.taboolaLogsEnabled) {
      return Logger.LOG_LEVEL_DEBUG.toString(); //Include all log levels
    }
    return Logger.LOG_LEVEL_ERROR.toString(); //Always log errors
  }

  String _getExternalRects() {
    String externalRects =
        RectsUtils.getExternalRects(nativeWindowRect, webViewRect);
//    Logger.log("externalRect: $externalRects");
    return externalRects;
  }

  String _isAlive() {
    return "true";
  }

  void _webViewDidScrollToTop() {
    taboolaWebViewDidScrollToTop();
    Logger.log("did scroll to top");
  }

  // Native calling JS
  void notifyExternalRects() {
    _notifyExternalRects(_getExternalRects());
  }

  void _notifyExternalRects(String externalRects) {
    taboolaBridge?.emit(sendNotifyExternalRects, [externalRects]);
  }

  void _fireClickEvent(url) {
    taboolaBridge?.emit(sendFireClickEvent, ['$url']);
  }
}

==> taboola_sdk-0.3.3.tar.gz: lib/classic/webview_manager.dart <==
import 'dart:async';
import 'dart:convert';
import 'package:flutter/material.dart';
import 'package:taboola_sdk/descriptors/application_descriptor.dart';
import 'package:taboola_sdk/descriptors/device_descriptor.dart';
import 'package:taboola_sdk/descriptors/sdk_descriptor.dart';
import 'package:taboola_sdk/classic/taboola_bridge.dart';
import 'package:taboola_sdk/utils/rects_utils.dart';
import 'package:taboola_sdk/utils/type_converter_utils.dart';
import 'package:webview_flutter/webview_flutter.dart';
import 'package:taboola_sdk/utils/logger.dart';
import 'package:taboola_sdk/native_plugin/taboola_sdk_plugin.dart';
import '../extra_properties_manager.dart';

const String onRenderSuccess = "onRenderSuccess";
const String onRenderFailure = "onRenderFailure";
const String onClick = "onClick";
const String onAttributionClick = "handleAttributionClick";
const String getDeviceData = "getDeviceData";
const String getLogLevel = "getLogLevel";
const String getExternalRects = "getExternalRects";
const String isAlive = "isAlive";
const String sendNotifyExternalRects = "notifyExternalRects";
const String sendFireClickEvent = "fireClickEvent";
const String webViewDidScrollToTop = "webViewDidScrollToTop";

class WebViewManager {
  late Rect webViewRect;
  late Rect nativeWindowRect;
  late TaboolaBridge taboolaBridge;
  late ExtraPropertiesManager _extraPropertiesManager;
  bool _didShow = false;

  final String flutterBridge =  "flutterBridge";
  final String actionKey =  "action";
  final String promiseIdKey =  "promiseId";
  final String parametersKey =  "parameters";

  // Taboola Classic Callbacks
  late void Function(String) didShow;
  late void Function(String, double) didResize;
  late void Function(String, String) didFailToLoad;
  late bool Function(String, String, String, bool) didClickOnItem;
  late void Function(String) openItemBrowser;
  late void Function() taboolaWebViewDidScrollToTop;

  WebViewManager(this._extraPropertiesManager);

  set controller(WebViewController controller) {
    taboolaBridge = TaboolaBridge(controller);
    controller.addJavaScriptChannel(flutterBridge,
        onMessageReceived: (JavaScriptMessage result) {
      Map<String, dynamic> messageMap = jsonDecode(result.message);
      String action = messageMap[actionKey];
      String promiseId = messageMap[promiseIdKey];
      List parameters = messageMap[parametersKey];
      _callNativeFunctionFromJS(action, promiseId, parameters);
    });
  }


  Future<void> _callNativeFunctionFromJS(
      String action, String promiseId, List parameters) async {
    Function func;
    switch (action) {
      case onRenderSuccess:
        func = _onRenderSuccess;
        break;
      case onRenderFailure:
        func = _onRenderFailure;
        break;
      case onClick:
        func = _onClick;
        break;
      case onAttributionClick:
        func = _onAttributionClick;
        break;
      case getDeviceData:
        func = _getDeviceData;
        break;
      case getLogLevel:
        func = _getLogLevel;
        break;
      case getExternalRects:
        func = _getExternalRects;
        break;
      case isAlive:
        func = _isAlive;
        break;
      case webViewDidScrollToTop:
        func = _webViewDidScrollToTop;
        break;
      default:
        return;
    }
    try {
      var returnValue = await Function.apply(func, parameters);
      taboolaBridge.resolvePromise(promiseId, returnValue);
    } catch (e) {
      Logger.logException("$func,$e");
    }
  }

  // JS calling Native
  void _onRenderSuccess(String placement, dynamic height) {
    didResize(placement, TypeConverterUtils.toDouble(height));
    if (!_didShow) {
      _didShow = true;
      didShow(placement);
    }
  }

  void _onRenderFailure(String placement, String error) {
    didFailToLoad(placement, error);
  }

  void _onClick(String title, String pageUrl, String loggerUrl, bool organic,
      String itemId, String placement) {
    bool taboolaHandleClick = true;
    bool publisherInputForHandleClick =
        didClickOnItem(placement, itemId, pageUrl, organic);

    if (organic) {
      taboolaHandleClick = publisherInputForHandleClick;
    }

    //if taboola needs to handle the click, open a browser with the link
    //if not, only fire click event
    if (taboolaHandleClick) {
      openItemBrowser(loggerUrl);
    } else {
      Logger.log(
          '_onClick: Publisher requested SDK not to handle click by returning false in taboolaDidClickOnItem event.');
      _fireClickEvent(loggerUrl);
    }
  }

  void _onAttributionClick(String url) {
    var encoded = Uri.encodeFull(url);
    openItemBrowser(encoded);
  }

  Future<String> _getDeviceData() async {
    List? screenSize = DeviceDescriptor.getScreenSize();

    var deviceData = {
      "verifier_enabled": false,
      "additional_data": {
        "sdkd": {
          "sdkt": SdkDescriptor.getSdkName(),
          "sdkv": SdkDescriptor.getSdkVersion(),
          "app": await ApplicationDescriptor.getAppName(),
          "model": await DeviceDescriptor.getDeviceModel(),
          "appv": await ApplicationDescriptor.getAppVersion(),
          "appid": await ApplicationDescriptor.getAppPackageName(),
          "lng": await DeviceDescriptor.getLanguageJson(),
          "os": await DeviceDescriptor.getDeviceOS(),
          "osv": await DeviceDescriptor.getDeviceSystemVersion(),
//          "kotlin": true, //FIXME: Exists in native SDK, implement post MVP
//          "androidx": true, //FIXME: Exists in native SDK, implement post MVP
//          "net": await DeviceDescriptor.getNetworkConnSpeed(), //FIXME: Re-enable once code is ripped from Connectivity plugin
          "mem": await DeviceDescriptor.getDeviceFreeMemory(),
          "scr": {"wd": screenSize?[0] ?? " ", "ht": screenSize?[1] ?? " "}
        }
      },
      "device": await DeviceDescriptor.getAdvertisingId(),
//      "allowNonOrganicClickOverride": false, //FIXME: Exists in native SDK, implement post MVP
//      "isUsedInTaboolaWidget": true, //FIXME: Exists in native SDK, implement post MVP
//      "taboolaConfig": "{}" //FIXME: Exists in native SDK, implement post MVP
    };
    await selfPopulateExtraProperties();

    Map<String, dynamic> dataToMobileLoader = new Map<String, dynamic>();
    dataToMobileLoader.addAll(
        _extraPropertiesManager.getExtraProperties());
    dataToMobileLoader.addAll(deviceData);

    String encoded = json.encode(dataToMobileLoader);
    Logger.log('deviceData: $encoded');

    return encoded;
  }

  Future<void> selfPopulateExtraProperties() async {
    //Consulting product manager, the user opt out will only be calculated by what the user selected in their operating system definitions. Not by publisher.
    _extraPropertiesManager.put(ExtraPropertiesManager.USER_OPT_OUT_KEY,
        await TaboolaSdkPlugin.isUserOptOut());
  }


  String _getLogLevel() {
    if (Logger.taboolaLogsEnabled) {
      return Logger.LOG_LEVEL_DEBUG.toString(); //Include all log levels
    }
    return Logger.LOG_LEVEL_ERROR.toString(); //Always log errors
  }

  String _getExternalRects() {
    String externalRects =
        RectsUtils.getExternalRects(nativeWindowRect, webViewRect);
    return externalRects;
  }

  String _isAlive() {
    return "true";
  }

  void _webViewDidScrollToTop() {
    taboolaWebViewDidScrollToTop();
    Logger.log("did scroll to top");
  }

  // Native calling JS
  void notifyExternalRects() {
    _notifyExternalRects(_getExternalRects());
  }

  void _notifyExternalRects(String externalRects) {
    taboolaBridge.emit(sendNotifyExternalRects, [externalRects]);
  }

  void _fireClickEvent(url) {
    taboolaBridge.emit(sendFireClickEvent, ['$url']);
  }
}
